@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2019, Free Software Foundation, Inc.
+Copyright (C) 2015-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -51,8 +51,8 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 
 
-/* In every node of the operand linked list, only one of the `filename' or
-   `data' should be non-NULL. Otherwise it will be a bug and will cause
+/* In every node of the operand linked list, only one of the 'filename' or
+   'data' should be non-NULL. Otherwise it will be a bug and will cause
    problems. All the operands operate on this premise. */
 struct operand
 {
@@ -81,6 +81,7 @@ struct arithmeticparams
   gal_data_t       refdata;  /* Container for information of the data.  */
   char          *globalhdu;  /* Single HDU for all inputs.              */
   uint8_t      onedasimage;  /* Write 1D outputs as an image not table. */
+  uint8_t     onedonstdout;  /* Write 1D outputs on stdout, not table.  */
   gal_data_t        *named;  /* List containing variables.              */
   size_t      tokencounter;  /* Counter for finding place in tokens.    */
 
