@@ -5,7 +5,7 @@ BuildProgram is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2019, Free Software Foundation, Inc.
+Copyright (C) 2017-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -45,7 +45,7 @@ buildprog_as_one_string(char *opt, gal_list_str_t *list)
   /* Only if we have a list. */
   if(list)
     {
-      /* For every node in the list, we want the `opt' and a space along with
+      /* For every node in the list, we want the 'opt' and a space along with
          the actual string. */
       for(tmp=list; tmp!=NULL; tmp=tmp->next)
         len += 1 + (opt ? strlen(opt) : 0) + strlen(tmp->v);
@@ -71,10 +71,11 @@ buildprog_as_one_string(char *opt, gal_list_str_t *list)
 int
 buildprog(struct buildprogparams *p)
 {
-  /* Note that the first node of `sourceargs' is the acutal source and the
+  /* Note that the first node of 'sourceargs' is the acutal source and the
      rest are arguments to be run later. */
   int retval;
   char *fullla;
+  char *ldflags=NULL, *cppflags=NULL;
   char *command, *optimize=NULL, *warning=NULL;
   char *include   = buildprog_as_one_string("-I", p->include);
   char *linkdir   = buildprog_as_one_string("-L", p->linkdir);
@@ -88,6 +89,13 @@ buildprog(struct buildprogparams *p)
       printf("---------------------------------\n");
     }
 
+  /* If environment should be read, read it. */
+  if(p->noenv==0)
+    {
+      ldflags=getenv("LDFLAGS");
+      cppflags=getenv("CPPFLAGS");
+    }
+
   /* Compiler options with values: */
   if(p->warning)
     if( asprintf(&warning,  "-W%s", p->warning)<0 )
@@ -96,25 +104,28 @@ buildprog(struct buildprogparams *p)
     if( asprintf(&optimize, "-O%s", p->optimize)<0 )
       error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
 
-  /* Libtool `.la' file: */
+  /* Libtool '.la' file: */
   if(p->la) fullla=p->la;
   else
     if( asprintf(&fullla, "%s/libgnuastro.la", LIBDIR)<0 )
       error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
 
   /* Write the full Libtool command into a string (to run afterwards). */
-  if( asprintf(&command, "%s -c \"%s %s %s%s --mode=link gcc %s %s "
-               "%s %s %s %s %s -I%s %s -o %s\"",
+  if( asprintf(&command, "%s -c \"%s %s %s%s --mode=link %s %s %s "
+               "%s %s %s %s %s %s %s -I%s %s -o %s\"",
                GAL_CONFIG_GNULIBTOOL_SHELL,
                GAL_CONFIG_GNULIBTOOL_EXEC,
                p->cp.quiet ? "--quiet" : "",
-               p->tag      ? "--tag="   : "",
+               p->tag      ? "--tag="  : "",
                p->tag      ? p->tag    : "",
+               p->cc,
                warning     ? warning   : "",
                p->debug    ? "-g"      : "",
                optimize    ? optimize  : "",
                include     ? include   : "",
+               cppflags    ? cppflags  : "",
                linkdir     ? linkdir   : "",
+               ldflags     ? ldflags   : "",
                p->sourceargs->v,
                linklib     ?linklib    : "",
                INCLUDEDIR,
