@@ -337,7 +337,7 @@ convertt(struct converttparams *p)
     /* Plain text: only one channel is acceptable. */
     case OUT_FORMAT_TXT:
       gal_checkset_writable_remove(p->cp.output, 0, p->cp.dontdelete);
-      gal_txt_write(p->chll, NULL, p->cp.output, 0);
+      gal_txt_write(p->chll, NULL, NULL, p->cp.output, 0);
       break;
 
     /* JPEG: */
