@@ -44,6 +44,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "velocity",
+      UI_KEY_VELOCITY,
+      "FLT",
+      0,
+      "Velocity of interest in km/s.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->velocity,
+      GAL_TYPE_FLOAT64,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "obsline",
       UI_KEY_OBSLINE,
@@ -303,6 +316,20 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       ui_add_to_single_value,
     },
+    {
+      "usedvelocity",
+      UI_KEY_USEDVELOCITY,
+      0,
+      0,
+      "Used velocity (in km/s) for this run.",
+      UI_GROUP_BASIC,
+      &p->specific,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_add_to_single_value,
+    },
 
 
 
