@@ -755,7 +755,7 @@ onecrop(struct onecropparams *crp)
       sprintf(regionkey, "%sPIX", basekeyname);
       gal_fits_key_list_add_end(&headers, GAL_TYPE_STRING, regionkey,
                                 0, region, 0, "Range of pixels used for "
-                                "this output.", 0, NULL);
+                                "this output.", 0, NULL, 0);
       gal_fits_key_write_in_ptr(&headers, ofp);
 
 
