@@ -362,7 +362,8 @@ ui_read_check_only_options(struct fitsparams *p)
     }
 
   /* Same for the extension-related options */
-  if( p->remove || p->copy || p->cut || p->numhdus || p->datasum )
+  if( p->remove || p->copy || p->cut || p->numhdus || p->datasum
+      || p->pixelscale )
     {
       /* A small sanity check. */
       if(p->mode!=FITS_MODE_INVALID)
@@ -588,7 +589,7 @@ ui_fill_fits_headerll(gal_list_str_t *input, gal_fits_list_key_t **output,
 
       /* Add it to the list of keywords. */
       gal_fits_key_list_add(output, type, keyname, 0, fvalue, vfree,
-                            comment, 0, unit);
+                            comment, 0, unit, 0);
       free(original);
     }
 
