@@ -83,8 +83,10 @@ enum option_keys_enum
   UI_KEY_MODESYMVALUE,
   UI_KEY_QUANTFUNC,
   UI_KEY_ASCIICFP,
+  UI_KEY_HISTOGRAM2D,
   UI_KEY_MIRROR,
   UI_KEY_NUMBINS,
+  UI_KEY_NUMBINS2,
   UI_KEY_NUMASCIIBINS,
   UI_KEY_ASCIIHEIGHT,
   UI_KEY_LOWERBIN,
@@ -105,6 +107,9 @@ enum option_keys_enum
   UI_KEY_SIGCLIPMEDIAN,
   UI_KEY_SIGCLIPMEAN,
   UI_KEY_SIGCLIPSTD,
+  UI_KEY_GREATEREQUAL2,
+  UI_KEY_LESSTHAN2,
+  UI_KEY_ONEBINSTART2,
 };
 
 
