@@ -3,9 +3,9 @@ Warp - Warp images using projective mapping.
 Warp is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -30,8 +30,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gnuastro/wcs.h>
 #include <gnuastro/fits.h>
+#include <gnuastro/array.h>
 #include <gnuastro/table.h>
 #include <gnuastro/threads.h>
+#include <gnuastro/pointer.h>
 
 #include <gnuastro-internal/timing.h>
 #include <gnuastro-internal/options.h>
@@ -75,15 +77,6 @@ doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" will warp/transform the "
 
 
 
-/* Option groups particular to this program. */
-enum program_args_groups
-{
-  ARGS_GROUP_WARPS = GAL_OPTIONS_GROUP_AFTER_COMMON,
-};
-
-
-
-
 
 
 
@@ -135,6 +128,7 @@ ui_initialize_options(struct warpparams *p,
 
         case GAL_OPTIONS_KEY_SEARCHIN:
         case GAL_OPTIONS_KEY_TABLEFORMAT:
+        case GAL_OPTIONS_KEY_STDINTIMEOUT:
           cp->coptions[i].flags=OPTION_HIDDEN;
           break;
         }
@@ -160,18 +154,18 @@ parse_opt(int key, char *arg, struct argp_state *state)
 {
   struct warpparams *p = state->input;
 
-  /* Pass `gal_options_common_params' into the child parser.  */
+  /* Pass 'gal_options_common_params' into the child parser.  */
   state->child_inputs[0] = &p->cp;
 
   /* In case the user incorrectly uses the equal sign (for example
-     with a short format or with space in the long format, then `arg`
+     with a short format or with space in the long format, then 'arg'
      start with (if the short version was called) or be (if the long
      version was called with a space) the equal sign. So, here we
      check if the first character of arg is the equal sign, then the
      user is warned and the program is stopped: */
   if(arg && arg[0]=='=')
-    argp_error(state, "incorrect use of the equal sign (`=`). For short "
-               "options, `=` should not be used and for long options, "
+    argp_error(state, "incorrect use of the equal sign ('='). For short "
+               "options, '=' should not be used and for long options, "
                "there should be no space between the option, equal sign "
                "and value");
 
@@ -218,7 +212,7 @@ parse_opt(int key, char *arg, struct argp_state *state)
 /**********      Modular matrix linked list       *************/
 /**************************************************************/
 /* Save the codes of the user's desired modular warpings into the linked
-   list. Because the types of these options are `GAL_TYPE_INVALID', this
+   list. Because the types of these options are 'GAL_TYPE_INVALID', this
    function will not be called when printing the full list of parameters
    and their values. */
 static void *
@@ -230,26 +224,32 @@ ui_add_to_modular_warps_ll(struct argp_option *option, char *arg,
   gal_data_t *new;
   struct warpparams *p=(struct warpparams *)params;
 
+  /* When an argument is necessary (note that '--align' doesn't take an
+     argument), make sure we actually have a string to parse. Also note
+     that if an argument is necessary, but none is given Argp will
+     automatically abort the program with an informative error. */
+  if(arg && *arg=='\0')
+    error(EXIT_FAILURE, 0, "empty string given to '--%s'", option->name);
 
   /* Parse the (possible) arguments. */
   if(option->key==UI_KEY_ALIGN)
     {
       /* For functions the standard checking isn't done, so first, we'll
          make sure that if we are in a configuration file (where
-         `arg!=NULL'), the value is either 0 or 1. */
+         'arg!=NULL'), the value is either 0 or 1. */
       if( arg && strcmp(arg, "0") && strcmp(arg, "1") )
-        error_at_line(EXIT_FAILURE, 0, filename, lineno, "the `--align' "
+        error_at_line(EXIT_FAILURE, 0, filename, lineno, "the '--align' "
                       "option takes no arguments. In a configuration file "
-                      "it can only have the values `1' or `0', indicating "
+                      "it can only have the values '1' or '0', indicating "
                       "if it should be used or not");
 
       /* Align doesn't take any values, but if called in a configuration
-         file with a value of `0', we should ignore it. */
+         file with a value of '0', we should ignore it. */
       if(arg && *arg=='0') return NULL;
 
       /* Allocate the data structure. */
-      new=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 0, NULL, NULL, 0,
-                         -1, NULL, NULL, NULL);
+      new=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 0, NULL, NULL, 0, -1, 1,
+                         NULL, NULL, NULL);
     }
   else new=gal_options_parse_list_of_numbers(arg, filename, lineno);
 
@@ -265,7 +265,7 @@ ui_add_to_modular_warps_ll(struct argp_option *option, char *arg,
                       "may be given, you can use multiple modular warpings");
       if(new->size!=4 && new->size!=9)
         error_at_line(EXIT_FAILURE, 0, filename, lineno, "only a 4 or 9 "
-                      "element `matrix' is currently acceptable. `%s' has "
+                      "element 'matrix' is currently acceptable. '%s' has "
                       "%zu elements", arg, new->size);
 
       /* Keep the matrix in the main structure. */
@@ -277,7 +277,7 @@ ui_add_to_modular_warps_ll(struct argp_option *option, char *arg,
          warpings. */
       if(new->size>2)
         error_at_line(EXIT_FAILURE, 0, filename, lineno, "%zu numbers "
-                      "given to the `%s' option. Modular warpings can "
+                      "given to the '%s' option. Modular warpings can "
                       "accept 2 numbers at the most currently (for 2D "
                       "datasets)", new->size, option->name);
 
@@ -286,9 +286,9 @@ ui_add_to_modular_warps_ll(struct argp_option *option, char *arg,
       if(option->key==UI_KEY_ROTATE)
         {
           if(new->size!=1)
-            error_at_line(EXIT_FAILURE, 0, filename, lineno, "the `rotate' "
+            error_at_line(EXIT_FAILURE, 0, filename, lineno, "the 'rotate' "
                       "option only takes one value (the angle of rotation). "
-                      "You have given: `%s'", arg);
+                      "You have given: '%s'", arg);
         }
       else if (option->key==UI_KEY_FLIP)
         {
@@ -296,9 +296,9 @@ ui_add_to_modular_warps_ll(struct argp_option *option, char *arg,
             {
               tmp=((double *)(new->array))[i];
               if(tmp!=0.0f && tmp!=1.0f)
-                error_at_line(EXIT_FAILURE, 0, filename, lineno, "`flip' "
-                              "only takes values of `1' and `0'. You have "
-                              "given `%s'", arg);
+                error_at_line(EXIT_FAILURE, 0, filename, lineno, "'flip' "
+                              "only takes values of '1' and '0'. You have "
+                              "given '%s'", arg);
             }
         }
 
@@ -340,17 +340,42 @@ ui_check_options_and_arguments(struct warpparams *p)
     {
       /* Make sure a HDU is given. */
       if( gal_fits_name_is_fits(p->inputname) && p->cp.hdu==NULL )
-        error(EXIT_FAILURE, 0, "no HDU specified, you can use the `--hdu' "
-              "(`-h') option and give it the HDU number (starting from "
+        error(EXIT_FAILURE, 0, "no HDU specified, you can use the '--hdu' "
+              "('-h') option and give it the HDU number (starting from "
               "zero), or extension name (generally, anything acceptable "
               "by CFITSIO)");
 
       /* Read the input image as double type and its WCS structure. */
-      p->input=gal_fits_img_read_to_type(p->inputname, p->cp.hdu,
-                                         GAL_TYPE_FLOAT64,
-                                         p->cp.minmapsize);
+      p->input=gal_array_read_one_ch_to_type(p->inputname, p->cp.hdu,
+                                             NULL, GAL_TYPE_FLOAT64,
+                                             p->cp.minmapsize,
+                                             p->cp.quietmmap);
+
+      /* Read the WCS and remove one-element wide dimension(s). */
       p->input->wcs=gal_wcs_read(p->inputname, p->cp.hdu, p->hstartwcs,
                                  p->hendwcs, &p->input->nwcs);
+      p->input->ndim=gal_dimension_remove_extra(p->input->ndim,
+                                                p->input->dsize,
+                                                p->input->wcs);
+
+      /* Currently Warp only works on 2D images. */
+      if(p->input->ndim!=2)
+        error(EXIT_FAILURE, 0, "input has %zu dimensions but Warp currently "
+              "only works on 2D datasets (images).\n\n"
+              "We do plan to add 3D functionality (see "
+              "https://savannah.gnu.org/task/?15729), so please get in "
+              "touch if you need it (any further interest, support or help "
+              "would be useful)", p->input->ndim);
+
+      /* Get basic WCS information. */
+      if(p->input->wcs)
+        {
+          p->pixelscale=gal_wcs_pixel_scale(p->input->wcs);
+          if(p->pixelscale==NULL)
+            error(EXIT_FAILURE, 0, "%s (hdu %s): the pixel scale couldn't "
+                  "be deduced from the WCS.", p->inputname, p->cp.hdu);
+          p->inwcsmatrix=gal_wcs_warp_matrix(p->input->wcs);
+        }
     }
   else
     error(EXIT_FAILURE, 0, "no input file is specified");
@@ -382,9 +407,9 @@ static void
 ui_error_no_warps()
 {
   error(EXIT_FAILURE, 0, "no warping specified, you can either use the "
-        "`--matrix' option for any low-level warp, or specify multipole "
-        "modular warpings with options like `--rotate', `--scale' and etc. "
-        "You can see the full list with the `--help' option");
+        "'--matrix' option for any low-level warp, or specify multipole "
+        "modular warpings with options like '--rotate', '--scale' and etc. "
+        "You can see the full list with the '--help' option");
 }
 
 
@@ -416,7 +441,7 @@ ui_matrix_prepare_raw(struct warpparams *p)
   if(p->matrix->size==4)
     {
       /* Allocate the final matrix. */
-      final=gal_data_malloc_array(GAL_TYPE_FLOAT64, 9, __func__, "final");
+      final=gal_pointer_allocate(GAL_TYPE_FLOAT64, 9, 0, __func__, "final");
 
       /* Fill in the final 3x3 matrix from the 2x2 matrix. */
       final[0]=in[0];    final[1]=in[1];   final[2]=0.0f;
@@ -432,8 +457,8 @@ ui_matrix_prepare_raw(struct warpparams *p)
   /* Correct the dimensional information, because the matrix was read as a
      single dimensional list of numbers. */
   free(p->matrix->dsize);
-  dsize=p->matrix->dsize=gal_data_malloc_array(GAL_TYPE_SIZE_T, 2, __func__,
-                                               "dsize");
+  dsize=p->matrix->dsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, 2, 0,
+                                               __func__, "dsize");
   dsize[0]=dsize[1]=3;
   p->matrix->ndim=2;
 }
@@ -447,103 +472,122 @@ ui_matrix_prepare_raw(struct warpparams *p)
 static void
 ui_matrix_make_align(struct warpparams *p, double *tmatrix)
 {
-  double A, *w, *ps, amatrix[4];
+  double A, *w, *P, x[4];
 
   /* Make sure the input image had a WCS structure. */
   if(p->input->wcs==NULL)
     error(EXIT_FAILURE, 0, "%s (hdu: %s): no WCS information present, "
-          "hence the `--align' option cannot be used", p->inputname,
+          "hence the '--align' option cannot be used", p->inputname,
           p->cp.hdu);
 
   /* Check if there is only two WCS axises: */
   if(p->input->wcs->naxis!=2)
     error(EXIT_FAILURE, 0, "the WCS structure of %s (hdu: %s) has %d "
-          "axises. For the `--align' option to operate it must be 2",
+          "axises. For the '--align' option to operate it must be 2",
           p->inputname, p->cp.hdu, p->input->wcs->naxis);
 
 
-  /* Find the pixel scale along the two dimensions. Note that we will be
-     using the scale along the image X axis for both values. */
-  w=gal_wcs_warp_matrix(p->input->wcs);
-  ps=gal_wcs_pixel_scale_deg(p->input->wcs);
+  /* For help in reading, use aliases for the WCS matrix and pixel scale.*/
+  P=p->pixelscale;
+  w=p->inwcsmatrix;
 
 
-  /* Lets call the given WCS orientation `W', the rotation matrix we want
-     to find as `X' and the final (aligned matrix) to have just one useful
-     value: `a' (which is the pixel scale):
+  /* Lets call the given WCS orientation 'W', the rotation matrix we want
+     to find as 'X' and the final (aligned matrix) 'P' (which is the pixel
+     scale):
 
-        x0  x1       w0  w1      -a  0
-        x2  x3   *   w2  w3   =   0  a
+          X            W             P
+        ------       ------      -------
+        x0  x1       w0  w1      -P0   0
+        x2  x3   *   w2  w3   =   0    P1
 
-     Let's open up the matrix multiplication, so we can find the `X'
-     elements as function of the `W' elements and `a'.
+     Let's open up the matrix multiplication, so we can find the 'X'
+     elements as function of the 'W' elements and 'a'.
 
-        x0*w0 + x1*w2 = -a                                         (1)
+        x0*w0 + x1*w2 = -P0                                        (1)
         x0*w1 + x1*w3 =  0                                         (2)
         x2*w0 + x3*w2 =  0                                         (3)
-        x2*w1 + x3*w3 =  a                                         (4)
+        x2*w1 + x3*w3 =  P1                                        (4)
 
      Let's bring the X with the smaller index in each equation to the left
      side:
 
-        x0 = (-w2/w0)*x1 - a/w0                                    (5)
+        x0 = (-w2/w0)*x1 - P0/w0                                   (5)
         x0 = (-w3/w1)*x1                                           (6)
         x2 = (-w2/w0)*x3                                           (7)
-        x2 = (-w3/w1)*x3 + a/w1                                    (8)
+        x2 = (-w3/w1)*x3 + P1/w1                                   (8)
 
     Using (5) and (6) we can find x0 and x1, by first eliminating x0:
 
-       (-w2/w0)*x1 - a/w0 = (-w3/w1)*x1 -> (w3/w1 - w2/w0) * x1 = a/w0
+       (-w2/w0)*x1 - P0/w0 = (-w3/w1)*x1  ->  (w3/w1 - w2/w0) * x1 = P0/w0
 
     For easy reading/writing, let's define: A = (w3/w1 - w2/w0)
 
-       --> x1 = a / w0 / A
+       --> x1 = P0 / w0 / A
        --> x0 = -1 * x1 * w3 / w1
 
     Similar to the above, we can find x2 and x3 from (7) and (8):
 
-       (-w2/w0)*x3 = (-w3/w1)*x3 + a/w1 -> (w3/w1 - w2/w0) * x3 = a/w1
+       (-w2/w0)*x3 = (-w3/w1)*x3 + P1/w1  ->  (w3/w1 - w2/w0) * x3 = P1/w1
 
-       --> x3 = a / w1 / A
+       --> x3 = P1 / w1 / A
        --> x2 = -1 * x3 * w2 / w0
 
-    Note that when the image is already aligned, a unity matrix should be
-    output.
-   */
+    Note that when the image is already aligned (off-diagonals are zero),
+    only the signs of the diagonal elements matter. */
   if( w[1]==0.0f && w[2]==0.0f )
     {
-      amatrix[0]=1.0f;   amatrix[1]=0.0f;
-      amatrix[2]=0.0f;   amatrix[3]=1.0f;
+      x[0] = w[0]<0 ? 1.0f : -1.0f;  /* Has to be negative. */
+      x[1] = 0.0f;
+      x[2] = 0.0f;
+      x[3] = w[3]>0 ? 1.0f : -1.0f;  /* Has to be positive. */
+    }
+  else if (w[0]==0.0f && w[3]==0.0f )
+    {
+      x[0] = 0.0f;
+      x[1] = w[1]<0 ? 1.0f : -1.0f;  /* Has to be negative. */
+      x[2] = w[2]>0 ? 1.0f : -1.0f;  /* Has to be positive. */
+      x[3] = 0.0f;
     }
   else
     {
       A = (w[3]/w[1]) - (w[2]/w[0]);
-      amatrix[1] = ps[0] / w[0] / A;
-      amatrix[3] = ps[0] / w[1] / A;
-      amatrix[0] = -1 * amatrix[1] * w[3] / w[1];
-      amatrix[2] = -1 * amatrix[3] * w[2] / w[0];
+      x[1] = P[0] / w[0] / A;
+      x[3] = P[1] / w[1] / A;
+      x[0] = -1 * x[1] * w[3] / w[1];
+      x[2] = -1 * x[3] * w[2] / w[0];
+
+      /* When the input matrix is flipped, the diagonal elements of the
+         necessary rotation will have a different sign. */
+      if(x[0]*x[3]<0)
+        if(!p->cp.quiet)
+          error(0, 0, "%s (hdu %s): WARNING (bug #55217)! The alignment may "
+                "not be correct! This is a recognized bug, we just haven't "
+                "had enough time to fix it yet, any help or support is "
+                "most welcome.\n\n"
+                "This may be due to the East and North not being left-hand "
+                "oriented. If this is the case, you can fix it by flipping "
+                "the image along the problematic axis (with '--flip=1,0' or "
+                "'--flip=0,1') and re-running Warp with '--align'. Note "
+                "that you can't yet call '--flip' in the same command as "
+                "'--align'",  p->inputname, p->cp.hdu);
     }
 
-
   /* For a check:
-  printf("ps: %e\n", ps);
+  printf("ps: (%e, %e)\n", P[0], P[1]);
   printf("w:\n");
   printf("  %.8e    %.8e\n", w[0], w[1]);
   printf("  %.8e    %.8e\n", w[2], w[3]);
   printf("x:\n");
-  printf("  %.8e    %.8e\n", amatrix[0], amatrix[1]);
-  printf("  %.8e    %.8e\n", amatrix[2], amatrix[3]);
+  printf("  %.8e    %.8e\n", x[0], x[1]);
+  printf("  %.8e    %.8e\n", x[2], x[3]);
   exit(0);
   */
 
   /* Put the matrix elements into the output array: */
-  tmatrix[0]=amatrix[0];  tmatrix[1]=amatrix[1]; tmatrix[2]=0.0f;
-  tmatrix[3]=amatrix[2];  tmatrix[4]=amatrix[3]; tmatrix[5]=0.0f;
-  tmatrix[6]=0.0f;        tmatrix[7]=0.0f;       tmatrix[8]=1.0f;
-
-  /* Clean up. */
-  free(w);
-  free(ps);
+  tmatrix[0]=x[0];  tmatrix[1]=x[1]; tmatrix[2]=0.0f;
+  tmatrix[3]=x[2];  tmatrix[4]=x[3]; tmatrix[5]=0.0f;
+  tmatrix[6]=0.0f;  tmatrix[7]=0.0f; tmatrix[8]=1.0f;
 }
 
 
@@ -553,7 +597,7 @@ ui_matrix_make_align(struct warpparams *p, double *tmatrix)
 static void
 ui_matrix_inplacw_multiply(double *in, double *with)
 {
-  /* `tin' will keep the values of the input array because we want to
+  /* 'tin' will keep the values of the input array because we want to
      write the multiplication result in the input array. */
   double tin[9]={in[0],in[1],in[2],in[3],in[4],in[5],in[6],in[7],in[8]};
 
@@ -584,7 +628,7 @@ ui_matrix_from_modular(struct warpparams *p)
 {
   gal_data_t *pop;
   size_t dsize[]={3,3};
-  double s, c, v1, v2, *final, module[9];
+  double s, c, v1, v2, *final, module[9]={1,0,0,  0,1,0,  0,0,1};
 
   /* Reverse the list of modular warpings to be in the same order as the
      user specified.*/
@@ -592,7 +636,8 @@ ui_matrix_from_modular(struct warpparams *p)
 
   /* Allocate space for the final matrix. */
   p->matrix=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 2, dsize, NULL, 0,
-                           p->cp.minmapsize, NULL, NULL, NULL);
+                           p->cp.minmapsize, p->cp.quietmmap,
+                           NULL, NULL, NULL);
   final=p->matrix->array;
 
   /* Fill in the final matrix to start with. */
@@ -611,7 +656,7 @@ ui_matrix_from_modular(struct warpparams *p)
       v2 = pop->size>1 ? ((double *)(pop->array))[1] : v1;
 
       /* Depending on the type of the modular warp do it. Recall that the
-         code for the warp, was stored in the `status' element of the data
+         code for the warp, was stored in the 'status' element of the data
          structure.*/
       switch(pop->status)
         {
@@ -622,9 +667,9 @@ ui_matrix_from_modular(struct warpparams *p)
         case UI_KEY_ROTATE:
           s = sin( v1 * M_PI / 180 );
           c = cos( v1 * M_PI / 180 );
-          module[0]=c;          module[1]=s;      module[2]=0.0f;
-          module[3]=-1.0f*s;    module[4]=c;      module[5]=0.0f;
-          module[6]=0.0f;       module[7]=0.0f;   module[8]=1.0f;
+          module[0]=c;          module[1]=-1.0f*s;  module[2]=0.0f;
+          module[3]=s;          module[4]=c;        module[5]=0.0f;
+          module[6]=0.0f;       module[7]=0.0f;     module[8]=1.0f;
           break;
 
         case UI_KEY_SCALE:
@@ -666,6 +711,16 @@ ui_matrix_from_modular(struct warpparams *p)
           break;
 
         case UI_KEY_TRANSLATE:
+          /* Translate cannot be called with '--centeroncorner'. */
+          if(p->centeroncorner)
+            error(EXIT_FAILURE, 0, "'--translate' and '--centeroncorner' "
+                  "(which is a type of translation) cannot be called "
+                  "together. To achieve the effect of --centeroncorner, "
+                  "start the warp steps with a translation of 0.5 to move "
+                  "the coordinate center to the corner of a pixel in each "
+                  "dimension");
+
+          /* Build the translation matrix. */
           module[0]=1.0f;       module[1]=0.0f;     module[2]=v1;
           module[3]=0.0f;       module[4]=1.0f;     module[5]=v2;
           module[6]=0.0f;       module[7]=0.0f;     module[8]=1.0f;
@@ -679,7 +734,7 @@ ui_matrix_from_modular(struct warpparams *p)
 
         default:
           error(EXIT_FAILURE, 0, "a bug! the code %d is not recognized as "
-                "a valid modular warp in `ui_matrix_from_modular', this is "
+                "a valid modular warp in 'ui_matrix_from_modular', this is "
                 "not your fault, something in the programming has gone "
                 "wrong. Please contact us at %s so we can correct it",
                 pop->status, PACKAGE_BUGREPORT);
@@ -712,7 +767,7 @@ ui_matrix_center_on_corner(struct warpparams *p)
   /* Translate them back into the proper FITS center. */
   ui_matrix_inplacw_multiply(before, after);
 
-  /* The final matrix is in `before', so put its values into the output
+  /* The final matrix is in 'before', so put its values into the output
      matrix. */
   b = before;
   df = (d=p->matrix->array) + p->matrix->size;
@@ -746,7 +801,7 @@ ui_matrix_finalize(struct warpparams *p)
     if(!isfinite(*d++))
       {
         ui_matrix_print(p->matrix->array);
-        error(EXIT_FAILURE, 0, "%f is not a `normal' number in the "
+        error(EXIT_FAILURE, 0, "%f is not a 'normal' number in the "
               "input matrix shown above", *(d-1));
       }
   while(d<df);
@@ -758,14 +813,14 @@ ui_matrix_finalize(struct warpparams *p)
     error(EXIT_FAILURE, 0, "the determinant of the given matrix "
           "is zero");
 
-  /* Check if the transformation is spatially invariant, in other words, if
-     it differs between differet regions of the output. If it doesn't we
-     can use this information for a more efficient processing. This is not
-     yet implemented. */
+  /* Not yet implemented: Check if the transformation is spatially
+     invariant, in other words, if it differs between differet regions of
+     the output. If it doesn't we can use this information for a more
+     efficient processing. */
 
    /* Make the inverse matrix: */
-  inv=p->inverse=gal_data_malloc_array(GAL_TYPE_FLOAT64, 9, __func__,
-                                       "p->inverse");
+  inv=p->inverse=gal_pointer_allocate(GAL_TYPE_FLOAT64, 9, 0, __func__,
+                                      "p->inverse");
   inv[0] = d[4]*d[8] - d[5]*d[7];
   inv[1] = d[2]*d[7] - d[1]*d[8];
   inv[2] = d[1]*d[5] - d[2]*d[4];
@@ -820,7 +875,7 @@ ui_set_suffix(struct warpparams *p)
   if(p->matrix==NULL && p->modularll==NULL) ui_error_no_warps();
 
   /* We only want the more meaningful suffix when the list is defined AND
-     when its only has one node (the `next' element is NULL). */
+     when its only has one node (the 'next' element is NULL). */
   if(p->matrix==NULL && p->modularll->next==NULL)
     switch(p->modularll->status)
       {
@@ -848,7 +903,7 @@ ui_set_suffix(struct warpparams *p)
       default:
         error(EXIT_FAILURE, 0, "a bug! please contact us at %s so we can "
               "fix the problem. The modular warp code %d is not recognized "
-              "in `ui_set_suffix'", PACKAGE_BUGREPORT, p->modularll->status);
+              "in 'ui_set_suffix'", PACKAGE_BUGREPORT, p->modularll->status);
         return NULL;
       }
   else
@@ -862,12 +917,12 @@ ui_set_suffix(struct warpparams *p)
 static void
 ui_preparations(struct warpparams *p)
 {
-  /* Set the output name. This needs to be done before `ui_finalize_matrix'
+  /* Set the output name. This needs to be done before 'ui_finalize_matrix'
      because that function will free the linked list of modular warpings
      which we will need to determine the suffix if no output name is
      specified. */
   if(p->cp.output)
-    gal_checkset_check_remove_file(p->cp.output, 0, p->cp.dontdelete);
+    gal_checkset_writable_remove(p->cp.output, 0, p->cp.dontdelete);
   else
     p->cp.output=gal_checkset_automatic_output(&p->cp, p->inputname,
                                                ui_set_suffix(p));
@@ -901,12 +956,13 @@ ui_preparations(struct warpparams *p)
 void
 ui_read_check_inputs_setup(int argc, char *argv[], struct warpparams *p)
 {
+  double *matrix;
   struct gal_options_common_params *cp=&p->cp;
 
 
-  /* Include the parameters necessary for argp from this program (`args.h')
-     and for the common options to all Gnuastro (`commonopts.h'). We want
-     to directly put the pointers to the fields in `p' and `cp', so we are
+  /* Include the parameters necessary for argp from this program ('args.h')
+     and for the common options to all Gnuastro ('commonopts.h'). We want
+     to directly put the pointers to the fields in 'p' and 'cp', so we are
      simply including the header here to not have to use long macros in
      those headers which make them hard to read and modify. This also helps
      in having a clean environment: everything in those headers is only
@@ -935,6 +991,10 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct warpparams *p)
   gal_options_print_state(&p->cp);
 
 
+  /* Prepare all the options as FITS keywords to write in output later. */
+  gal_options_as_fits_keywords(&p->cp);
+
+
   /* Check that the options and arguments fit well with each other. Note
      that arguments don't go in a configuration file. So this test should
      be done after (possibly) printing the option values. */
@@ -948,8 +1008,9 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct warpparams *p)
   /* Everything is ready, notify the user of the program starting. */
   if(!p->cp.quiet)
     {
-      double *matrix=p->matrix->array;
-      printf(PROGRAM_NAME" started on %s", ctime(&p->rawtime));
+      matrix=p->matrix->array;
+      printf(PROGRAM_NAME" "PACKAGE_VERSION" started on %s",
+             ctime(&p->rawtime));
       printf(" Using %zu CPU thread%s\n", p->cp.numthreads,
              p->cp.numthreads==1 ? "." : "s.");
       printf(" Input: %s (hdu: %s)\n", p->inputname, p->cp.hdu);
@@ -993,6 +1054,8 @@ ui_free_report(struct warpparams *p, struct timeval *t1)
   free(p->cp.output);
   gal_data_free(p->input);
   gal_data_free(p->matrix);
+  if(p->pixelscale) free(p->pixelscale);
+  if(p->inwcsmatrix) free(p->inwcsmatrix);
 
   /* Report how long the operation took. */
   if(!p->cp.quiet)
