@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2018, Free Software Foundation, Inc.
+Copyright (C) 2016-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -55,7 +55,7 @@ binary_erode_dilate_2d_4con(gal_data_t *input, int dilate0_erode1)
   /* Do a sanity check: */
   if(dilate0_erode1!=1 && dilate0_erode1!=0)
     error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s so we can "
-          "fix this problem. The value to `dilate0_erode1' is %u while it "
+          "fix this problem. The value to 'dilate0_erode1' is %u while it "
           "should be 0 or 1", __func__, PACKAGE_BUGREPORT, dilate0_erode1);
 
   /* Set the foreground and background values. */
@@ -143,7 +143,7 @@ binary_erode_dilate_2d_8con(gal_data_t *input, unsigned char dilate0_erode1)
 
   /* Set the foreground and background values: */
   if(dilate0_erode1==0) {f=1; b=0;}
-  else         {f=0; b=1;}
+  else                  {f=0; b=1;}
 
   /* Check the 4 corners: */
   if(byt[0]==b && (byt[1]==f
@@ -221,15 +221,53 @@ binary_erode_dilate_2d_8con(gal_data_t *input, unsigned char dilate0_erode1)
 
 
 
-/* Erode a binary dataset any number of times. If `inplace' is given a
-   value of `1', then do the erosion within the already allocated space,
+/* This is a general erosion and dilation function. It is less efficient
+   than the more specialized cases above. */
+static void
+binary_erode_dilate_general(gal_data_t *input, unsigned char dilate0_erode1,
+                            int connectivity)
+{
+  uint8_t f, b, *pt, *fpt, *byt=input->array;
+  size_t i, *dinc=gal_dimension_increment(input->ndim, input->dsize);
+
+  /* Do a sanity check: */
+  if(dilate0_erode1!=1 && dilate0_erode1!=0)
+    error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s so we can fix "
+          "this problem. The value to dilate0_erode1 is %u while it should "
+          "be 0 or 1", __func__, PACKAGE_BUGREPORT, dilate0_erode1);
+
+  /* Set the foreground and background values: */
+  if(dilate0_erode1==0) {f=1; b=0;}
+  else                  {f=0; b=1;}
+
+  /* Go over the neighbors of each pixel. */
+  for(i=0;i<input->size;++i)
+    if(byt[i]==b)
+      GAL_DIMENSION_NEIGHBOR_OP(i, input->ndim, input->dsize, connectivity,
+                                dinc,{
+                                  if(byt[i]!=GAL_BINARY_TMP_VALUE
+                                     && byt[nind]==f)
+                                    byt[i]=GAL_BINARY_TMP_VALUE;
+                                });
+
+  /* Set all the changed pixels to the proper values: */
+  fpt=(pt=byt)+input->size;
+  do *pt = *pt==GAL_BINARY_TMP_VALUE ? f : *pt; while(++pt<fpt);
+}
+
+
+
+
+
+/* Erode a binary dataset any number of times. If 'inplace' is given a
+   value of '1', then do the erosion within the already allocated space,
    otherwise, allocate a new array and save the result into that.
 
    This function will only work on the elements with a value of 1 or 0. It
    will leave all the rest unchanged. Also note that it only works on
-   `uint8_t' type datasets. So if the input doesn't have that type, it is
+   'uint8_t' type datasets. So if the input doesn't have that type, it is
    going to copy it this type and return the newlyallocated dataset. So
-   when the input's type isn't `uint8_t', `inplace' is irrelevant. */
+   when the input's type isn't 'uint8_t', 'inplace' is irrelevant. */
 static gal_data_t *
 binary_erode_dilate(gal_data_t *input, size_t num, int connectivity,
                     int inplace, int d0e1)
@@ -241,13 +279,13 @@ binary_erode_dilate(gal_data_t *input, size_t num, int connectivity,
   /* Currently this only works on blocks. */
   if(input->block)
     error(EXIT_FAILURE, 0, "%s: currently only works on a fully "
-          "allocated block of memory, but the input is a tile (its `block' "
+          "allocated block of memory, but the input is a tile (its 'block' "
           "element is not NULL)", __func__);
 
   /* Set the dataset to work on. */
   binary = ( (inplace && input->type==GAL_TYPE_UINT8)
-             ? input :
-             gal_data_copy_to_new_type(input, GAL_TYPE_UINT8) );
+             ? input
+             : gal_data_copy_to_new_type(input, GAL_TYPE_UINT8) );
 
   /* Go over every element and do the erosion. */
   switch(binary->ndim)
@@ -264,6 +302,11 @@ binary_erode_dilate(gal_data_t *input, size_t num, int connectivity,
           }
       break;
 
+    case 3:
+      for(counter=0;counter<num;++counter)
+        binary_erode_dilate_general(binary, d0e1, connectivity);
+      break;
+
     default:
       error(EXIT_FAILURE, 0, "%s: currently doesn't work on %zu "
             "dimensional datasets", __func__, binary->ndim);
@@ -309,8 +352,8 @@ gal_binary_open(gal_data_t *input, size_t num, int connectivity,
   /* First do the necessary number of erosions. */
   out=gal_binary_erode(input, num, connectivity, inplace);
 
-  /* If `inplace' was called, then `out' is the same as `input', if it
-     wasn't, then `out' is a newly allocated array. In any case, we should
+  /* If 'inplace' was called, then 'out' is the same as 'input', if it
+     wasn't, then 'out' is a newly allocated array. In any case, we should
      dilate in the same allocated space. */
   gal_binary_dilate(input, num, connectivity, 1);
 
@@ -354,7 +397,7 @@ gal_binary_connected_components(gal_data_t *binary, gal_data_t **out,
 
   /* Two small sanity checks. */
   if(binary->type!=GAL_TYPE_UINT8)
-    error(EXIT_FAILURE, 0, "%s: the input data set type must be `uint8'",
+    error(EXIT_FAILURE, 0, "%s: the input data set type must be 'uint8'",
           __func__);
   if(binary->block)
     error(EXIT_FAILURE, 0, "%s: currently, the input data structure to "
@@ -369,13 +412,13 @@ gal_binary_connected_components(gal_data_t *binary, gal_data_t **out,
 
       /* Make sure the given dataset has the same size as the input. */
       if( gal_dimension_is_different(binary, lab) )
-        error(EXIT_FAILURE, 0, "%s: the `binary' and `out' datasets must "
+        error(EXIT_FAILURE, 0, "%s: the 'binary' and 'out' datasets must "
               "have the same size", __func__);
 
-      /* Make sure it has a `int32' type. */
+      /* Make sure it has a 'int32' type. */
       if( lab->type!=GAL_TYPE_INT32 )
-        error(EXIT_FAILURE, 0, "%s: the `out' dataset must have `int32' type"
-              "but the array you have given is `%s' type", __func__,
+        error(EXIT_FAILURE, 0, "%s: the 'out' dataset must have 'int32' type"
+              "but the array you have given is '%s' type", __func__,
               gal_type_name(lab->type, 1));
 
       /* Reset all its values to zero. */
@@ -384,7 +427,8 @@ gal_binary_connected_components(gal_data_t *binary, gal_data_t **out,
   else
     lab=*out=gal_data_alloc(NULL, GAL_TYPE_INT32, binary->ndim,
                             binary->dsize, binary->wcs, 1,
-                            binary->minmapsize, NULL, "labels", NULL);
+                            binary->minmapsize, binary->quietmmap,
+                            NULL, "labels", NULL);
 
 
   /* Initialize the labels array. If we have blank pixels in the byt
@@ -447,6 +491,97 @@ gal_binary_connected_components(gal_data_t *binary, gal_data_t **out,
 
 
 
+/* Put the indexs of connected labels in a list of 'gal_data_t's, each with
+   a one-dimensional array that has the indexs of that connected
+   component.*/
+#define BINARY_CONINDEX_VAL 2
+gal_data_t *
+gal_binary_connected_indexs(gal_data_t *binary, int connectivity)
+{
+  uint8_t *b, *bf;
+  gal_data_t *lines=NULL;
+  size_t p, i, onelabnum, *onelabarr;
+  gal_list_sizet_t *Q=NULL, *onelab=NULL;
+  size_t *dinc=gal_dimension_increment(binary->ndim, binary->dsize);
+
+  /* Small sanity checks. */
+  if(binary->type!=GAL_TYPE_UINT8)
+    error(EXIT_FAILURE, 0, "%s: the input data set type must be 'uint8'",
+          __func__);
+  if(binary->block)
+    error(EXIT_FAILURE, 0, "%s: currently, the input data structure to "
+          "must not be a tile", __func__);
+
+  /* Go over all the pixels and do a breadth-first search. */
+  b=binary->array;
+  for(i=0;i<binary->size;++i)
+    /* A pixel that has already been recorded is given a value of
+       'BINARY_CONINDEX_VAL'. */
+    if( b[i]==1 )
+      {
+        /* Add this pixel to the queue of pixels to work with. */
+	b[i]=BINARY_CONINDEX_VAL;
+        gal_list_sizet_add(&Q, i);
+        gal_list_sizet_add(&onelab, i);
+
+        /* While a pixel remains in the queue, continue labelling and
+           searching for neighbors. */
+        while(Q!=NULL)
+          {
+            /* Pop an element from the queue. */
+            p=gal_list_sizet_pop(&Q);
+
+            /* Go over all its neighbors and add them to the list if they
+               haven't already been labeled. */
+            GAL_DIMENSION_NEIGHBOR_OP(p, binary->ndim, binary->dsize,
+                                      connectivity, dinc,
+              {
+                if( b[nind]==1 )
+                  {
+		    b[nind]=BINARY_CONINDEX_VAL;
+                    gal_list_sizet_add(&Q, nind);
+		    gal_list_sizet_add(&onelab, nind);
+                  }
+              } );
+          }
+
+	/* Parsing has finished, put all the indexs into an array. */
+	onelabarr=gal_list_sizet_to_array(onelab, 1, &onelabnum);
+	gal_list_data_add_alloc(&lines, onelabarr, GAL_TYPE_SIZE_T, 1,
+				&onelabnum, NULL, 0, -1, 1, NULL, NULL, NULL);
+
+	/* Clean up. */
+	gal_list_sizet_free(onelab);
+	onelab=NULL;
+      }
+
+  /* Reverse the order. */
+  gal_list_data_reverse(&lines);
+
+  /* For a check
+  {
+    gal_data_t *test=lines->next;
+    size_t *b, *bf;
+    bf=(b=test->array)+test->size;
+    do printf("%zu\n", *b++);
+    while(b<bf);
+    exit(0);
+  }
+  */
+
+  /* Set all the '2' values back to '1'. */
+  bf=(b=binary->array)+binary->size;
+  do if(*b==BINARY_CONINDEX_VAL) *b=1; while(++b<bf);
+
+  /* Clean up and return the total number. */
+  free(dinc);
+  return lines;
+}
+
+
+
+
+
 /* Given an adjacency matrix (which should be binary), find the number of
    connected objects and return an array of new labels for each old
    label.  */
@@ -462,8 +597,8 @@ gal_binary_connected_adjacency_matrix(gal_data_t *adjacency,
 
   /* Some small sanity checks. */
   if(adjacency->type != GAL_TYPE_UINT8)
-    error(EXIT_FAILURE, 0, "%s: input must have type `uint8'. However, the "
-          "input dataset has type of `%s'", __func__,
+    error(EXIT_FAILURE, 0, "%s: input must have type 'uint8'. However, the "
+          "input dataset has type of '%s'", __func__,
           gal_type_name(adjacency->type, 1));
 
   if(adjacency->ndim != 2)
@@ -479,7 +614,8 @@ gal_binary_connected_adjacency_matrix(gal_data_t *adjacency,
 
   /* Allocate (and clear) the output datastructure. */
   newlabs_d=gal_data_alloc(NULL, GAL_TYPE_INT32, 1, &num, NULL, 1,
-                           adjacency->minmapsize, NULL, NULL, NULL);
+                           adjacency->minmapsize, adjacency->quietmmap,
+                           NULL, NULL, NULL);
   newlabs=newlabs_d->array;
 
 
@@ -586,15 +722,16 @@ binary_make_padded_inverse(gal_data_t *input, gal_data_t **outtile)
      the connected components and any hole, will get a value larger than
      1. */
   inv=gal_data_alloc(NULL, GAL_TYPE_UINT8, input->ndim, dsize, NULL, 0,
-                     input->minmapsize, "INVERSE", "binary", NULL);
+                     input->minmapsize, input->quietmmap,
+                     "INVERSE", "binary", NULL);
   memset(inv->array, 1, inv->size);
 
 
   /* Define a tile to fill the central regions of the inverse. */
   startind=gal_dimension_coord_to_index(input->ndim, inv->dsize, startcoord);
   tile=gal_data_alloc(gal_pointer_increment(inv->array, startind, inv->type),
-                      inv->type, input->ndim, input->dsize, NULL, 0, 0, NULL,
-                      NULL, NULL);
+                      inv->type, input->ndim, input->dsize, NULL, 0, 0, 0,
+                      NULL, NULL, NULL);
   *outtile=tile;
   tile->block=inv;
 
@@ -629,8 +766,8 @@ gal_binary_holes_label(gal_data_t *input, int connectivity,
 
   /* A small sanity check. */
   if( input->type != GAL_TYPE_UINT8 )
-    error(EXIT_FAILURE, 0, "%s: input must have `uint8' type, but its "
-          "input dataset has `%s' type", __func__,
+    error(EXIT_FAILURE, 0, "%s: input must have 'uint8' type, but its "
+          "input dataset has '%s' type", __func__,
           gal_type_name(input->type, 1));
 
 
@@ -648,13 +785,13 @@ gal_binary_holes_label(gal_data_t *input, int connectivity,
      we should invert the respective pixel. To do it, we'll use the tile
      that was defined before, just change its block and array.*/
   tile->array=gal_tile_block_relative_to_other(tile, holelabs);
-  tile->block=holelabs; /* has to be after correcting `tile->array'. */
+  tile->block=holelabs; /* has to be after correcting 'tile->array'. */
 
 
-  /* The type of the tile is already known (it is `int32_t') and we have no
-     output/other, so we'll just put `int' as a place-holder. In this way
+  /* The type of the tile is already known (it is 'int32_t') and we have no
+     output/other, so we'll just put 'int' as a place-holder. In this way
      we can avoid the switch statement of GAL_TILE_PARSE_OPERATE, and
-     directly use the workhorse macro `GAL_TILE_PO_OISET'. */
+     directly use the workhorse macro 'GAL_TILE_PO_OISET'. */
   lab=(holelabs)->array;
   GAL_TILE_PO_OISET(int32_t, int, tile, NULL, 0, 0, {
       *lab++ = ( *i
@@ -675,7 +812,7 @@ gal_binary_holes_label(gal_data_t *input, int connectivity,
      from the start, effectively removing the paddings. Therefore, ee will
      just correct the sizes and we won't bother actually re-allocating the
      array size in memory. According to the GNU C library's description
-     after `realloc': "In several allocation implementations, making a
+     after 'realloc': "In several allocation implementations, making a
      block smaller sometimes necessitates copying it, so it can fail if no
      other space is available.". The extra padding is only 2 pixels wide,
      thus, the extra space is negligible compared to the actual array. So
@@ -713,7 +850,7 @@ gal_binary_holes_label(gal_data_t *input, int connectivity,
 
    3. Since we had a 2 pixel padding on the edges of the image, we
       know for sure that all labeled regions with a label of 1 are
-      actually connected `holes' in the input image.
+      actually connected 'holes' in the input image.
 
       Any pixel with a label larger than 1, is therefore a bounded
       hole that is not 8-connected to the rest of the holes.  */
@@ -725,11 +862,15 @@ gal_binary_holes_fill(gal_data_t *input, int connectivity, size_t maxsize)
   size_t numholes, *sizes;
   gal_data_t *inv, *tile, *holelabs=NULL;
 
-  /* A small sanity check. */
+  /* Small sanity checks. */
   if( input->type != GAL_TYPE_UINT8 )
-    error(EXIT_FAILURE, 0, "%s: input must have `uint8' type, but its "
-          "input dataset has `%s' type", __func__,
+    error(EXIT_FAILURE, 0, "%s: input must have 'uint8' type, but its "
+          "input dataset has '%s' type", __func__,
           gal_type_name(input->type, 1));
+  if(connectivity<1 || connectivity>input->ndim)
+    error(EXIT_FAILURE, 0, "%s: connectivity value %d is not acceptable. "
+          "It has to be between 1 and the number of input's dimensions "
+          "(%zu)", __func__, connectivity, input->ndim);
 
 
   /* Make the inverse image. */
@@ -745,7 +886,7 @@ gal_binary_holes_fill(gal_data_t *input, int connectivity, size_t maxsize)
      that was defined before, just change its block and array.*/
   in=input->array;
   tile->array=gal_tile_block_relative_to_other(tile, holelabs);
-  tile->block=holelabs; /* has to be after correcting `tile->array'. */
+  tile->block=holelabs; /* has to be after correcting 'tile->array'. */
 
   /* If the user wants to only fill holes to a certain size, then remove
      those with a larger size. */
@@ -767,10 +908,10 @@ gal_binary_holes_fill(gal_data_t *input, int connectivity, size_t maxsize)
       free(sizes);
     }
 
-  /* The type of the tile is already known (it is `int32_t') and we have no
-     output, so we'll just put `int' as a place-holder. In this way we can
+  /* The type of the tile is already known (it is 'int32_t') and we have no
+     output, so we'll just put 'int' as a place-holder. In this way we can
      avoid the switch statement of GAL_TILE_PARSE_OPERATE, and directly use
-     the workhorse macro `GAL_TILE_PO_OISET'. */
+     the workhorse macro 'GAL_TILE_PO_OISET'. */
   GAL_TILE_PO_OISET(int32_t, int, tile, NULL, 0, 0, {
       *in = *i>1 && *i!=GAL_BLANK_INT32 ? 1 : *in;
       ++in;
