@@ -13,6 +13,7 @@
 # Original author:
 #     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
+# Copyright (C) 2015-2020, Free Software Foundation, Inc.
 #
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
@@ -56,7 +57,8 @@ if [ ! -f $cat      ]; then echo "$cat does not exist.";   exit 77; fi
 # Actual test script
 # ==================
 #
-# `check_with_program' can be something like `Valgrind' or an empty
+# 'check_with_program' can be something like Valgrind or an empty
 # string. Such programs will execute the command if present and help in
 # debugging when the developer doesn't have access to the user's system.
-$check_with_program $execname $cat --naxis=100,100 --crpix=-99,1 --individual
+$check_with_program $execname $cat --mergedsize=100,100 --crpix=-99,1 \
+                    --individual
